@@ -14,48 +14,6 @@ from absbox.local.util import mkTag,DC,mkTs,query,consolStmtByDate,aggStmtByDate
 from absbox.local.component import *
 
 
-def mkAssetRate(x):
-    match x:
-        case ["固定",r]:
-            return mkTag(("Fix",r))
-        case ["浮动",r,{"基准":idx,"利差":spd,"重置频率":p}]:
-            return mkTag(("Floater",[idx,spd,r,freqMap[p],None]))
-
-def mkAmortPlan(x):
-    match x:
-        case "等额本息" | "Level" :
-            return mkTag("Level")
-        case "等额本金" | "Even" :
-            return mkTag("Even")
-        case _ :
-            raise RuntimeError(f"Failed to match AmortPlan {x}")
-
-
-def mkAsset(x):
-    _typeMapping = {"等额本息": "Level", "等额本金": "Even"}
-    _statusMapping = {"正常": mkTag(("Current")), "违约": mkTag(("Defaulted",None))}
-    match x:
-        case ["按揭贷款"
-            ,{"放款金额": originBalance, "放款利率": originRate, "初始期限": originTerm
-                  ,"频率": freq, "类型": _type, "放款日": startDate}
-            ,{"当前余额": currentBalance
-             ,"当前利率": currentRate
-             ,"剩余期限": remainTerms
-             ,"状态": status}]:
-            return mkTag(("Mortgage",[
-                                      {"originBalance": originBalance,
-                                      "originRate": mkAssetRate(originRate),
-                                      "originTerm": originTerm,
-                                      "period": freqMap[freq],
-                                      "startDate": startDate,
-                                      "prinType": mkAmortPlan(_type)
-                                      } |mkTag("OriginalInfo"),
-                                     currentBalance,
-                                     currentRate,
-                                     remainTerms,
-                                     _statusMapping[status]
-                    ]))
-
 def mkCf(x):
     if len(x)==0:
         return None
@@ -123,24 +81,6 @@ def mkLiq(x):
             return mkTag(("PV", [df, r]))
 
 
-def mkCallOptions(x):
-    match x:
-        case {"资产池余额": bal}:
-            return mkTag(("PoolBalance", bal))
-        case {"债券余额": bal}:
-            return mkTag(("PoolBalance", bal))
-        case {"资产池余额剩余比率": factor}:
-            return mkTag(("PoolFactor", factor))
-        case {"债券余额剩余比率": factor}:
-            return mkTag(("PoolFactor", factor))
-        case {"指定日之后": d}:
-            return mkTag(("AfterDate", d))
-        case {"任意满足": xs}:
-            return mkTag(("Or", xs))
-        case {"全部满足": xs}:
-            return mkTag(("And", xs))
-
-
 def mkAssumption(x):
     match x:
         case {"CPR": cpr} if isinstance(cpr,list):
@@ -188,47 +128,6 @@ def mkLiq(x):
             return mkTag(("PV", [df, r]))
 
 
-def mkCallOptions(x):
-    match x:
-        case {"资产池余额": bal}:
-            return mkTag(("PoolBalance", bal))
-        case {"债券余额": bal}:
-            return mkTag(("BondBalance", bal))
-        case {"资产池余额剩余比率": factor}:
-            return mkTag(("PoolFactor", factor))
-        case {"债券余额剩余比率": factor}:
-            return mkTag(("BondFactor", factor))
-        case {"指定日之后": d}:
-            return mkTag(("AfterDate", d))
-        case {"任意满足": xs}:
-            return mkTag(("Or", xs))
-        case {"全部满足": xs}:
-            return mkTag(("And", xs))
-
-
-def mkAssumption(x):
-    match x:
-        case {"CPR": cpr} if isinstance(cpr, list):
-            return mkTag(("PrepaymentCPRCurve", cpr))
-        case {"CPR": cpr} :
-            return mkTag(("PrepaymentCPR", cpr))
-        case {"CPR调整": [*cprAdj,eDate]} :
-            return mkTag(("PrepaymentFactors" , mkTs("FactorCurveClosed",[cprAdj,eDate])))
-        case {"CDR": cdr}:
-            return mkTag(("DefaultCDR", cdr))
-        case {"CDR调整": [*cdrAdj,eDate]} :
-            return mkTag(("DefaultFactors" , mkTs("FactorCurveClosed",[cdrAdj,eDate])))
-        case {"回收": (rr, rlag)}:
-            return mkTag(("Recovery", (rr, rlag)))
-        case {"利率": [idx, rate]} if isinstance(rate, float):
-            return mkTag(("InterestRateConstant", [idx, rate]))
-        case {"利率": [idx, *rateCurve]}:
-            return mkTag(("InterestRateCurve", [idx, *rateCurve]))
-        case {"清仓": opts}:
-            return mkTag(("CallWhen",[mkCallOptions(co) for co in opts]))
-        case {"停止": d}:
-            return mkTag(("StopRunBy",d))
-
 def mkCustom(x):
     match x:
         case {"常量":n}:
@@ -255,20 +154,6 @@ def mk(x):
         case ["清仓回购", calls]:
             return mkCall(calls)
 
-def readStatus(s):
-    if "," in s:
-        return s.split(",")[1]
-    else:
-        return mkTag("Amortizing")
-
-class 时间点(Enum):
-    回收后 = "BeginCollectionWF"
-    回收动作后 = "EndCollectionWF"
-    分配前 = "BeginDistributionWF"
-    分配后 = "EndDistributionWF"
-
-
-
 
 @dataclass
 class 信贷ABS:
@@ -286,7 +171,6 @@ class 信贷ABS:
     触发事件: dict = None
     状态:str = "摊销"
 
-
     @classmethod
     def load(cls,p):
         with open(p,'rb') as _f:
@@ -332,8 +216,7 @@ class 信贷ABS:
             "pool":{"assets": [mkAsset(x) for x in self.资产池.get('清单',[])]
                 , "asOfDate": self.日期['封包日']
                 , "issuanceStat": readIssuance(self.资产池)
-                , "futureCf":mkCf(self.资产池.get('归集表', []))
-                },
+                , "futureCf":mkCf(self.资产池.get('归集表', []))},
             "bonds": functools.reduce(lambda result, current: result | current
                                       , [mk(['债券', bn, bo]) for (bn, bo) in self.债券]),
             #"waterfall": {f"DistributionDay {status['tag']}":list(distsflt)
@@ -368,7 +251,10 @@ class 信贷ABS:
             for (_k, _p) in self.流动性支持.items():
                 _providers[_k] = mkLiqProvider(_k, ( _p | {"起始日": self.日期["起息日"]}))
             _r["liqProvider"] = _providers
-        return _r  # ,ensure_ascii=False)
+
+        _dealType = identify_deal_type(_r)
+
+        return mkTag((_dealType,_r))
 
     def _get_bond(self, bn):
         for _bn,_bo in self.债券:
