@@ -12,12 +12,6 @@ datePattern = {"月末":"MonthEnd"
               ,"每月":"DayOfMonth"
               ,"每周":"DayOfWeek"}
 
-class 频率(Enum):
-    每月 = 12
-    每季度 = 4
-    每半年 = 2
-    每年 = 1
-
 
 freqMap = {"每月": "Monthly"
     , "每季度": "Quarterly"
@@ -323,6 +317,8 @@ def mkRateReset(x):
             return mkTag(("ByInterval", [freqMap[interval], None]))
         case {"重置月份": monthOfYear}:
             return mkTag(("MonthOfYear", monthOfYear))
+        case _:
+            raise RuntimeError(f"Failed to match:{x}: mkRateReset")
 
       
 
@@ -379,7 +375,7 @@ def mkBnd(bn,x):
                           , "bndDueIntDate": None }}
 
         case _:
-            raise RuntimeError(f"Failed to match bond:{bn},{x}")
+            raise RuntimeError(f"Failed to match bond:{bn},{x}:mkBnd")
 
 def mkLiqMethod(x):
     match x:
@@ -389,7 +385,8 @@ def mkLiqMethod(x):
             return mkTag(("BalanceFactor2",[a,b,c]))
         case ["贴现|违约",a,b]:
             return mkTag(("PV",[a,b]))
-
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:mkLiqMethod")
 
 
 def mkFeeCapType(x):
@@ -398,11 +395,17 @@ def mkFeeCapType(x):
             return mkTag(("DuePct",pct))
         case {"应计费用上限": amt}:
             return mkTag(("DueCapAmt",amt))
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:mkFeeCapType")
+
 
 def mkPDA(x):
     match x:
         case {"公式": ds}:
             return mkTag(("DS",mkDs(ds)))
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:mkPDA")
+
 
 def mkAccountCapType(x):
     match x:
@@ -410,6 +413,9 @@ def mkAccountCapType(x):
             return mkTag(("DuePct",pct))
         case {"金额上限": amt}:
             return mkTag(("DueCapAmt",amt))
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:mkAccountCapType")
+            
 
 def mkTransferLimit(x):
     match x:
@@ -422,7 +428,7 @@ def mkTransferLimit(x):
         case {"公式": formula}:
             return mkTag(("DS",mkDs(formula)))
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkTransferLimit")
 
 
 
@@ -475,7 +481,7 @@ def mkAction(x):
         case ["流动性支持报酬",source, target]| ["liqRepayResidual",source, target]:
             return mkTag(("LiqYield",[None, source, target]))
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkAction")
 
 
 def mkWaterfall2(x):
@@ -503,7 +509,7 @@ def mkStatus(x):
         case "结束":
             return mkTag(("Ended"))
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkStatus")
 
 def mkWhenTrigger(x):
     match x:
@@ -516,7 +522,7 @@ def mkWhenTrigger(x):
         case "分配后":
             return "EndDistributionWF"
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkWhenTrigger")
 
 
 def mkThreshold(x):
@@ -530,7 +536,7 @@ def mkThreshold(x):
         case "<=":
             return "EqBelow"
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkThreshold")
 
 def _rateTypeDs(x):
     h = x[0]
@@ -567,7 +573,7 @@ def mkTrigger(x):
         case ["一直",b]:
             return mkTag(("Always",b))
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkTrigger")
 
 
 def mkTriggerEffect(x):
@@ -581,7 +587,7 @@ def mkTriggerEffect(x):
         case ["结果",*efs]:
             return mkTag(("TriggerEffects",[mkTriggerEffect(e) for e in efs]))
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkTriggerEffect")
 
 #def mkActionWhen(x):
 #    match x:
@@ -618,6 +624,162 @@ def mkWaterfall(r, x):
         case "回款日" | "回款后" :
             _w_tag = f"EndOfPoolCollection"
         case _:
-            raise RuntimeError(f"Failed to match :{x}")
+            raise RuntimeError(f"Failed to match :{x}:mkWaterfall")
     r[_w_tag] = itertools.chain.from_iterable([mkWaterfall2(_a) for _a in _v])
     return mkWaterfall(r, x)
+
+
+def mkAssetRate(x):
+    match x:
+        case ["固定",r]:
+            return mkTag(("Fix",r))
+        case ["浮动",r,{"基准":idx,"利差":spd,"重置频率":p}]:
+            return mkTag(("Floater",[idx,spd,r,freqMap[p],None]))
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:mkAssetRate")
+
+def mkAmortPlan(x)->dict:
+    match x:
+        case "等额本息" | "Level" :
+            return mkTag("Level")
+        case "等额本金" | "Even" :
+            return mkTag("Even")
+        case "先息后本" | "I_P" :
+            return mkTag("I_P")
+        case "等本等费" | "F_P" :
+            return mkTag("F_P")
+        case _ :
+            raise RuntimeError(f"Failed to match AmortPlan {x}:mkAmortPlan")
+
+
+def mkAsset(x):
+    _statusMapping = {"正常": mkTag(("Current")), "违约": mkTag(("Defaulted",None))}
+    match x:
+        case ["按揭贷款"
+            ,{"放款金额": originBalance, "放款利率": originRate, "初始期限": originTerm
+                  ,"频率": freq, "类型": _type, "放款日": startDate}
+            ,{"当前余额": currentBalance
+             ,"当前利率": currentRate
+             ,"剩余期限": remainTerms
+             ,"状态": status}]:
+            return mkTag(("Mortgage",[
+                                      {"originBalance": originBalance,
+                                      "originRate": mkAssetRate(originRate),
+                                      "originTerm": originTerm,
+                                      "period": freqMap[freq],
+                                      "startDate": startDate,
+                                      "prinType": mkAmortPlan(_type)
+                                      } |mkTag("MortgageOriginalInfo"),
+                                     currentBalance,
+                                     currentRate,
+                                     remainTerms,
+                                     _statusMapping[status]]))
+        case ["贷款"
+            ,{"放款金额": originBalance, "放款利率": originRate, "初始期限": originTerm
+                  ,"频率": freq, "类型": _type, "放款日": startDate}
+            ,{"当前余额": currentBalance
+             ,"当前利率": currentRate
+             ,"剩余期限": remainTerms
+             ,"状态": status}]:
+            return mkTag(("PersonalLoan",[
+                                      {"originBalance": originBalance,
+                                      "originRate": mkAssetRate(originRate),
+                                      "originTerm": originTerm,
+                                      "period": freqMap[freq],
+                                      "startDate": startDate,
+                                      "prinType": mkAmortPlan(_type)
+                                      } | mkTag("LoanOriginalInfo"),
+                                     currentBalance,
+                                     currentRate,
+                                     remainTerms,
+                                     _statusMapping[status]]))       
+        case ["分期"
+            ,{"放款金额": originBalance, "放款费率": originRate, "初始期限": originTerm
+                  ,"频率": freq, "类型": _type, "放款日": startDate}
+            ,{"当前余额": currentBalance
+             ,"剩余期限": remainTerms
+             ,"状态": status}]:
+            return mkTag(("Installment",[
+                                      {"originBalance": originBalance,
+                                      "originRate": mkAssetRate(originRate),
+                                      "originTerm": originTerm,
+                                      "period": freqMap[freq],
+                                      "startDate": startDate,
+                                      "prinType": mkAmortPlan(_type)
+                                      } | mkTag("LoanOriginalInfo"),
+                                     currentBalance,
+                                     remainTerms,
+                                     _statusMapping[status]]))       
+ 
+
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:mkAsset")
+
+
+def identify_deal_type(x):
+    match x:
+        case {"pool":{"assets":[{'tag':'PersonalLoan'},*rest]}}:
+            return "LDeal"
+        case {"pool":{"assets":[{'tag':'Mortgage'},*rest]}} :
+            return "MDeal"
+        case {"pool":{"assets":[{'tag':'Installment'},*rest]}} :
+            return "IDeal"
+        case _ :
+            return "MDeal"
+
+def mkCallOptions(x):
+    match x:
+        case {"资产池余额": bal}:
+            return mkTag(("PoolBalance", bal))
+        case {"债券余额": bal}:
+            return mkTag(("BondBalance", bal))
+        case {"资产池余额剩余比率": factor}:
+            return mkTag(("PoolFactor", factor))
+        case {"债券余额剩余比率": factor}:
+            return mkTag(("BondFactor", factor))
+        case {"指定日之后": d}:
+            return mkTag(("AfterDate", d))
+        case {"任意满足": xs}:
+            return mkTag(("Or", xs))
+        case {"全部满足": xs}:
+            return mkTag(("And", xs))
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:mkCallOptions")
+            
+
+def mkAssumption(x) -> dict:
+    match x:
+        case {"CPR": cpr} if isinstance(cpr, list):
+            return mkTag(("PrepaymentVec", cpr))
+        case {"CDR": cdr} if isinstance(cdr, list):
+            return mkTag(("DefaultVec", cdr))
+        case {"CPR": cpr} :
+            return mkTag(("PrepaymentCPR", cpr))
+        case {"CPR调整": [*cprAdj,eDate]} :
+            return mkTag(("PrepaymentFactors" , mkTs("FactorCurveClosed",[cprAdj,eDate])))
+        case {"CDR": cdr}:
+            return mkTag(("DefaultCDR", cdr))
+        case {"CDR调整": [*cdrAdj,eDate]} :
+            return mkTag(("DefaultFactors" , mkTs("FactorCurveClosed",[cdrAdj,eDate])))
+        case {"回收": (rr, rlag)}:
+            return mkTag(("Recovery", (rr, rlag)))
+        case {"利率": [idx, rate]} if isinstance(rate, float):
+            return mkTag(("InterestRateConstant", [idx, rate]))
+        case {"利率": [idx, *rateCurve]}:
+            return mkTag(("InterestRateCurve", [idx, *rateCurve]))
+        case {"清仓": opts}:
+            return mkTag(("CallWhen",[mkCallOptions(co) for co in opts]))
+        case {"停止": d}:
+            return mkTag(("StopRunBy",d))
+        case _ :
+            raise RuntimeError(f"Failed to match {x}:Assumption")
+
+def mkPool(x):
+    mapping = {"LDeal":"LPool","MDeal":"MPool","IDeal":"IPool"}
+    match x:
+        case {"清单":assets,"封包日":coday}:
+            _pool = {"assets": [mkAsset(a) for a in assets],"asOfDate": coday}
+            _pool_asset_type = identify_deal_type({"pool":_pool})
+            return mkTag((mapping[_pool_asset_type], _pool))
+        case _:
+            raise RuntimeError(f"Failed to match {x}:mkPool")
